/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "adc.h"
#include "dma.h"
#include "tim.h"
#include "usb_device.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "utils.h"
#include "command.h"
#include "armscope_settings.h"
#include "usbd_cdc_if.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
static uint16_t adcData[4*1024*2] = {0};
static volatile uint16_t *lastAdcData = NULL;
uint8_t txFifo[32*1024];
volatile int txFifoWpos = 0;
volatile int txFifoRpos = 0;
enum { TRANSMIT_SIZE = (2*1024) };

void TransmitNextData(void);

static const char szId[] = "miniscope_v2g 20210501";
int identified = 0;

struct S_ARMSCOPE_SETTINGS settings;

enum E_ACTION_REQUEST action = REQUEST_NONE;
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

static void processSamples(volatile const uint16_t *data, unsigned int dataCnt)  {
#if 1
    {
        // verify sampling
        static unsigned int cnt = 0;
        cnt++;
        if (cnt == 1000)
        {
            volatile unsigned int now = HAL_GetTick();
            now = HAL_GetTick();
        }
    }
#endif
    volatile int rpos = txFifoRpos;
    volatile int wpos = txFifoWpos;

    if (rpos > wpos && wpos + dataCnt >= rpos)
        return;

    for (unsigned int i=0; i<dataCnt; i++)
    {
        txFifo[wpos++] = data[i] & 0xFF;
    }
    if (wpos >= ARRAY_SIZE(txFifo))
    {
        wpos = 0;
    }
    txFifoWpos = wpos;
}

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
    /* USER CODE BEGIN 1 */

    /* USER CODE END 1 */

    /* MCU Configuration--------------------------------------------------------*/

    /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
    HAL_Init();

    /* USER CODE BEGIN Init */

    /* USER CODE END Init */

    /* Configure the system clock */
    SystemClock_Config();

    /* USER CODE BEGIN SysInit */

    /* USER CODE END SysInit */

    /* Initialize all configured peripherals */
    MX_GPIO_Init();
    MX_DMA_Init();
    MX_ADC1_Init();
    MX_TIM3_Init();
    MX_USB_DEVICE_Init();
    /* USER CODE BEGIN 2 */
    HAL_ADC_Start_DMA(&hadc1, (uint32_t*)adcData, ARRAY_SIZE(adcData));
    HAL_TIM_PWM_Start(&htim3, TIM_CHANNEL_3);
    /* USER CODE END 2 */

    /* Infinite loop */
    /* USER CODE BEGIN WHILE */
    while (1)
    {
        /* USER CODE END WHILE */
        uint32_t now = HAL_GetTick();
        now >>= 9;
        if (now & 0x01)
        {
            HAL_GPIO_WritePin(LED_GPIO_Port, LED_Pin, GPIO_PIN_SET);
        }
        else
        {
            HAL_GPIO_WritePin(LED_GPIO_Port, LED_Pin, GPIO_PIN_RESET);
        }


        if (action == REQUEST_IDENTIFY)
        {
            action = REQUEST_NONE;
            //Buzzer_Off();
            //buzz += 30000;
            HAL_Delay(100);
            CDC_Transmit_FS((unsigned char*)szId, strlen(szId));
            //// delay - make sure that reply would not be overwritten by new samples
            HAL_Delay(100);
            identified = 1;
        }

        if (CDC_Tx_Idle() == 0)
        {
            TransmitNextData();
        }
        /* USER CODE BEGIN 3 */
    }
    /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
    RCC_OscInitTypeDef RCC_OscInitStruct = {0};
    RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

    /** Configure the main internal regulator output voltage
    */
    __HAL_RCC_PWR_CLK_ENABLE();
    __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE2);
    /** Initializes the CPU, AHB and APB busses clocks
    */
    RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
    RCC_OscInitStruct.HSEState = RCC_HSE_ON;
    RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
    RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
    RCC_OscInitStruct.PLL.PLLM = 25;
    RCC_OscInitStruct.PLL.PLLN = 288;
    RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV4;
    RCC_OscInitStruct.PLL.PLLQ = 6;
    if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
    {
        Error_Handler();
    }
    /** Initializes the CPU, AHB and APB busses clocks
    */
    RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                                  |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
    RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
    RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
    RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
    RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

    if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
    {
        Error_Handler();
    }
}

/* USER CODE BEGIN 4 */

void HAL_ADC_ConvHalfCpltCallback(ADC_HandleTypeDef* hadc)
{
    //printf("HT\r\n");
    lastAdcData = adcData;
    processSamples(lastAdcData, ARRAY_SIZE(adcData)/2);
}

void HAL_ADC_ConvCpltCallback(ADC_HandleTypeDef* hadc)
{
    //printf("FT\r\n");
    _Static_assert(ARRAY_SIZE(adcData) % 2 == 0, "adcData size must be even");
    lastAdcData = adcData + (ARRAY_SIZE(adcData)/2);
    processSamples(lastAdcData, ARRAY_SIZE(adcData)/2);
}

void HAL_ADC_ErrorCallback(ADC_HandleTypeDef *hadc)
{
    Error_Handler();
}

void OnUsbDataRx(uint8_t* dataIn, uint8_t length)
{
    enum E_ACTION_REQUEST tmp;

    if (length != 64)
    {
        return;
    }

    tmp = ExecuteCmd(dataIn, length, &settings);
    if (tmp != REQUEST_NONE)
    {
        action = tmp;
    }
}

void TransmitNextData(void)
{
    volatile int wpos = txFifoWpos;
    if (wpos - txFifoRpos >= TRANSMIT_SIZE ||
         ((txFifoRpos > wpos) && (ARRAY_SIZE(txFifo) - txFifoRpos >= TRANSMIT_SIZE)))
    {
        int rpos = txFifoRpos;
        CDC_Transmit_FS(&txFifo[rpos], TRANSMIT_SIZE);
        if (txFifoRpos + TRANSMIT_SIZE > ARRAY_SIZE(txFifo))
        {
            assert_failed(__FILE__, __LINE__);
        }
        else if (txFifoRpos + TRANSMIT_SIZE == ARRAY_SIZE(txFifo))
        {
            txFifoRpos = 0;
        }
        else
        {
            txFifoRpos += TRANSMIT_SIZE;
        }
    }
}


/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
    /* USER CODE BEGIN Error_Handler_Debug */
    while (1)
    {
        HAL_GPIO_TogglePin(LED_GPIO_Port, LED_Pin);
        HAL_Delay(50);
    }
    /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
    /* USER CODE BEGIN 6 */
    while (1)
    {
        HAL_GPIO_TogglePin(LED_GPIO_Port, LED_Pin);
        HAL_Delay(50);
    }
    /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
